using NinjaTrader.Cbi;
using NinjaTrader.Core;
using NinjaTrader.Data;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.NinjaScript.DrawingTools;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Media;
using System.Xml.Serialization;

// This indicator plots the prior two day's settlements as a horizontal lines.  Currently,
// only the CME Index Futures settlement time is used for determining the settlement price.  It is the
// same time for many other futures products, which is the closing price at 3:15 PM Central Time.
//
// Author:  Steven Lynn
// Email:   smlguard24-ninjatrader@yahoo.com

namespace NinjaTrader.NinjaScript.Indicators
{
	public class SettlementLines : Indicator
	{
		private double settlementPrice;
		private double priorSettlementPrice;

		private SessionIterator sessionIterator;

		private readonly List<int> newSessionBarIdxArr = new List<int>();
		private DateTime currentDate = Globals.MinDate;
		private DateTime cacheSessionDate = Globals.MinDate;
		private DateTime cacheSessionEnd = Globals.MinDate;
		private DateTime cacheTime;
		private DateTime sessionDateTmp = Globals.MinDate;


		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description = "Futures Settlement Lines - RTH Session Close Price";
				Name = "Settlement Lines";
				Calculate = Calculate.OnPriceChange;
				IsAutoScale = false;
				IsOverlay = true;
				IsSuspendedWhileInactive = true;
				DisplayInDataBox = true;
				DrawOnPricePanel = true;
				PaintPriceMarkers = true;
				ScaleJustification = ScaleJustification.Right;

				AddPlot(new Stroke(Brushes.Cyan, 2), PlotStyle.Hash, "Settlement");
				AddPlot(new Stroke(Brushes.HotPink, 2), PlotStyle.Hash, "Prior Settlement");
			}
			else if (State == State.Configure)
			{
				AddDataSeries(Instrument.FullName, new BarsPeriod { BarsPeriodType = BarsPeriodType.Day, Value = 1 }, "CME US Index Futures RTH", false);
			}
			else if (State == State.Historical)
			{
				// Displays a message if the bartype is not intraday
				if (!Bars.BarsType.IsIntraday)
				{
					Draw.TextFixed(this, "NinjaScriptInfo", "Futures Settlement Lines Intraday Only Error", TextPosition.BottomRight);
					Log("Futures Settlement Lines only supports Intraday charts", LogLevel.Error);
				}
			}
			else if (State == State.DataLoaded)
			{
				//stores the sessions once bars are ready, but before OnBarUpdate is called
				sessionIterator = new SessionIterator(BarsArray[1]);
			}
		}

		protected override void OnBarUpdate()
		{
			if (BarsInProgress != 0)
				return;

			DateTime lastBarTimeStamp = GetLastBarSessionDate(Times[0][0]);

			if (currentDate != Globals.MinDate && lastBarTimeStamp != currentDate)
			{
				priorSettlementPrice = settlementPrice;
				settlementPrice = DaySeriesClose[0];
			}
			currentDate = lastBarTimeStamp;

			if (priorSettlementPrice != 0)
			{
				PriorSettlementPlotValues[0] = priorSettlementPrice;
			}

			if (settlementPrice != 0)
			{
				SettlementPlotValues[0] = settlementPrice;
			}
		}

		#region Misc
		private DateTime GetLastBarSessionDate(DateTime time)
		{
			// Check the time[0] against the previous session end
			if (time > cacheSessionEnd)
			{
				if (Bars.BarsType.IsIntraday)
				{
					// Make use of the stored session iterator to find the next session...
					sessionIterator.GetNextSession(time, true);
					// Store the actual session's end datetime as the session
					cacheSessionEnd = sessionIterator.ActualSessionEnd;
					// We need to convert that time from the session to the users time zone settings
					sessionDateTmp = TimeZoneInfo.ConvertTime(cacheSessionEnd.AddSeconds(-1), Globals.GeneralOptions.TimeZoneInfo, Bars.TradingHours.TimeZoneInfo).Date;
				}
				else
				{
					sessionDateTmp = time.Date;
				}
			}

			if (sessionDateTmp != cacheSessionDate)
			{
				if (newSessionBarIdxArr.Count == 0 || newSessionBarIdxArr.Count > 0 && CurrentBar > newSessionBarIdxArr[newSessionBarIdxArr.Count - 1])
				{
					newSessionBarIdxArr.Add(CurrentBar);
				}
				cacheSessionDate = sessionDateTmp;
			}
			return sessionDateTmp;
		}
		#endregion


		#region Properties

		[Browsable(false)]
		[XmlIgnore()]
		public PriceSeries DaySeriesClose
		{
			get { return Closes[1]; }
		}

		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> SettlementPlotValues
		{
			get { return Values[0]; }
		}

		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> PriorSettlementPlotValues
		{
			get { return Values[1]; }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private SettlementLines[] cacheSettlementLines;
		public SettlementLines SettlementLines()
		{
			return SettlementLines(Input);
		}

		public SettlementLines SettlementLines(ISeries<double> input)
		{
			if (cacheSettlementLines != null)
				for (int idx = 0; idx < cacheSettlementLines.Length; idx++)
					if (cacheSettlementLines[idx] != null &&  cacheSettlementLines[idx].EqualsInput(input))
						return cacheSettlementLines[idx];
			return CacheIndicator<SettlementLines>(new SettlementLines(), input, ref cacheSettlementLines);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.SettlementLines SettlementLines()
		{
			return indicator.SettlementLines(Input);
		}

		public Indicators.SettlementLines SettlementLines(ISeries<double> input )
		{
			return indicator.SettlementLines(input);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.SettlementLines SettlementLines()
		{
			return indicator.SettlementLines(Input);
		}

		public Indicators.SettlementLines SettlementLines(ISeries<double> input )
		{
			return indicator.SettlementLines(input);
		}
	}
}

#endregion
